function map = diskmap(v,f)

% compute disk conformal parameterization of a simply-connected open surface
%% Initialization

parameter.north = 5;
parameter.south = 100;
parameter.threshold = 0.00001;

TR = TriRep(f,v);
B = freeBoundary(TR);
bdy_index = B(:,1);
bdy_length = edge_length(v, bdy_index);
partial_edge_sum = zeros(length(bdy_length),1);

for i = 2:length(bdy_length)
    for j = 1 : i-1
    partial_edge_sum(i) = partial_edge_sum(i) + bdy_length(j);
    end
end
theta = 2*pi.*partial_edge_sum/sum(bdy_length)+2*pi*0.01;
bdy = exp(theta*1i)';

% Disk harmonic map
Oper = createOperator(v',f');
z = mesh2disk(v,f,bdy_index,bdy, Oper.laplacian);
map = [real(z),imag(z),ones(length(z),1)]; map_prev = map;

if sum(sum(isnan(map))) ~= 0
    % use tutte embedding instead
    map = disktuttemap(v,f,bdy_index,bdy); map_prev = map;
    z = map(:,1) + 1i*map(:,2);
end

%% North Pole iteration
% all boundary points will be mapped to the real line

mu = bc_metric(map', f', v', 3); 
mu_v = Oper.f2v*mu;
bdy_index_temp = [bdy_index(2:end);bdy_index(1)];
[~, least] = min(abs(mu_v(bdy_index))+abs(mu_v(bdy_index_temp)));
z = z* exp(-1i*(angle(z(bdy_index(least)))+angle(z(bdy_index(least+1)))/2));
g = 1i*(1 + z)./(1 - z);

% fix the points near the puncture, i.e. near z = 1
[~, ind] = sort(-real(z));
fixed = setdiff(ind(1:max(round(length(v)/parameter.north),min(100,length(z)))), bdy_index);
fixed = [fixed; find(real(g) == max(real(g))); find(real(g) == min(real(g)))];

P = [real(g),imag(g),ones(length(g),1)];
mu = bc_metric(P', f', v', 3); 

% compute updated x coordinate
target = P(fixed,1);
A = generalized_laplacian2D(P',f',mu);
b = -A(:,fixed)*target;
b(fixed) = target;
A(fixed,:) = 0; A(:,fixed) = 0;
A = A + sparse(fixed,fixed,ones(length(fixed),1), size(A,1), size(A,2));
x = A\b;

% compute updated y coordinate
target = P(fixed,2); 
fixed = [fixed;bdy_index];
target = [target;zeros(length(bdy_index),1)];
A = generalized_laplacian2D(P',f',mu);
b = -A(:,fixed)*target;
b(fixed) = target;
A(fixed,:) = 0; A(:,fixed) = 0;
A = A + sparse(fixed,fixed,ones(length(fixed),1), size(A,1), size(A,2));
y = A\b;

g_new = complex(x,y);
z_new = (g_new - 1i)./(g_new + 1i);
map = [real(z_new), imag(z_new), 1+0*z_new];

if sum(sum(isnan(map))) ~= 0 % use the old result
    map = map_prev;
    z_new = map(:,1) + 1i*map(:,2);
end
%% Reflection

f_temp = f + length(v);
a = sort(bdy_index + length(v));
for i = length(a):-1:1
    f_temp(f_temp == a(i)) = a(i) - length(v);
    f_temp = f_temp - (f_temp >a(i));
end
f_filled = [f;fliplr(f_temp)];

z_filled = [z_new;1./conj(z_new)];
z_filled(bdy_index + length(v)) = [];

energy = mean(abs(bc_metric([real(z_new),imag(z_new),1+0*z_new]', f', v', 3)));
iteration_count = 2; 

map_opt = map;
finalno = 0;

%% South pole iteration
while iteration_count<=50
%%
    mu = bc_metric([real(z_new),imag(z_new),1+0*z_new]', f', v', 3);
    mu_filled = [mu;1/3*((z_new(f(:,1))./(conj(z_new(f(:,1))))).^2 + (z_new(f(:,2))./(conj(z_new(f(:,2))))).^2 + (z_new(f(:,3))./(conj(z_new(f(:,3))))).^2).*conj(mu)./abs(((z_new(f(:,1))./(conj(z_new(f(:,1))))).^2 + (z_new(f(:,2))./(conj(z_new(f(:,2))))).^2 + (z_new(f(:,3))./(conj(z_new(f(:,3))))).^2))];

    % fix the points near infinity
    [~, ind] = sort(-abs(z_filled));
    fixed2 = ind(1:max(round(length(z_filled)/parameter.south),min(100,length(z))));
    [map_filled,~] = lbs([real(z_filled),imag(z_filled),1+0*z_filled]',f_filled',mu_filled,fixed2,[real(z_filled(fixed2)), imag(z_filled(fixed2))]);

    z_big = complex(map_filled(1,:),map_filled(2,:))';
    z_final = z_big(1:length(v));
    
    % normalization
    z_final = z_final - mean(z_final); % move centroid to zero
    if max(abs(z_final))>1
        z_final = z_final/(max(abs(z_final))); % map it into unit circle
    end
    mu_temp = bc_metric([real(z_final),imag(z_final),1+0*z_final]',f',v',3);
    [map_temp,~] = lbs([real(z_final),imag(z_final),1+0*z_final]',f',mu_temp,bdy_index,[real(z_final(bdy_index)./abs(z_final(bdy_index))), imag(z_final(bdy_index)./abs(z_final(bdy_index)))]);
    map_temp = map_temp';
    z_new = map_temp(:,1) + 1i*map_temp(:,2);

    z_filled = [z_new;1./conj(z_new)]; % double count!
    z_filled(bdy_index + length(v)) = [];

    map = [real(z_new), imag(z_new), 1+0*z_new];
    
    if sum(sum(isnan(map))) ~= 0 % use the old result
        map = map_opt;
        if mod(finalno,2) == 1
            % flip once
            map(:,1) = -map(:,1);
        end
        return;
    end

    energy2 = mean(abs(bc_metric(map', f', v', 3)));
    
    map_opt = map;
    finalno = iteration_count;
  
    if abs(energy-energy2) <= parameter.threshold
        break;
    end
    
    energy = energy2;
    iteration_count = iteration_count+1;
end
map = map_opt;
if mod(finalno,2) == 1
    % flip once
    map(:,1) = -map(:,1);
end